from __future__ import annotations

__all__ = ["__version__"]

__version__ = "1.7.1"

# Optionally expose key functions at package root
# from .generator import generate_meve  # noqa: F401
# from .verifier import verify_meve     # noqa: F401
